import copy
import currypy
import logging

import pomsets.resource as ResourceModule
import pypatterns.filter as FilterModule
import pomsets.error as ErrorModule

"""
assumptions: directed acyclic graph
"""

class GraphObject (ResourceModule.ResourceReference, ResourceModule.Struct):

    ATTRIBUTES = [
        'graph'
        ]

    def __init__(self, id=None, graph=None):
        ResourceModule.ResourceReference.__init__(self, id=id)
        ResourceModule.Struct.__init__(self)
        self.graph(graph)
        pass

    def __ne__(self, other):
        """
        implements != for all the subclasses
        """
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result        

    def __deepcopy__(self, memo):
        if self in memo:
            return memo[self]
        
        copyId = self.id()
        graph = self.graph()
        copyGraph = memo.get(graph, graph)
        result = self.__class__()
        result.id(copyId)
        result.graph(copyGraph)
        copyResource = copy.deepcopy(self.resource(), memo)
        result.resource(copyResource)
        
        memo[self] = result
        return result
    
    
    # END class GraphObject
    pass

class Node (GraphObject):

    ATTRIBUTES = GraphObject.ATTRIBUTES + []

    
    @staticmethod
    def staticIsContainer(object):
        return False
    
    def __init__(self, id=None, graph=None):
        GraphObject.__init__(self, id=id, graph=graph)
        pass

    
    def __repr__(self):
        return "%s" % self.id()

    
    def __eq__(self, other):
        for difference in self.getDifferencesWith(other):
            return False
        return True

    def getDifferencesWith(self, other):

        if not isinstance(other, self.__class__):
            yield "self is of class %s, other is of class %s" % (self.__class__, other.__class__)

        #if not self.getKeyForEquivalenceTesting() == other.getKeyForEquivalenceTesting():
        #    yield "self has equivalence key %s, other has %s" % (self.getKeyForEquivalenceTesting(),
        #                                                         other.getKeyForEquivalenceTesting())
            
        selfIncomingEdges = self.getEdges(self.incomingEdgeFilter())
        otherIncomingEdges = other.getEdges(other.incomingEdgeFilter())
        if not len(selfIncomingEdges) == len(otherIncomingEdges):
            yield 'self has %s incoming edges, other has %s' % (len(selfIncomingEdges),
                                                                len(otherIncomingEdges))
        else:
            for selfEdge, otherEdge in zip(sorted(selfIncomingEdges, Edge.nameComparator),
                                           sorted(otherIncomingEdges, Edge.nameComparator)):
                selfEdgeName = selfEdge.name()
                otherEdgeName = otherEdge.name()
                if not selfEdgeName == otherEdgeName:
                    yield 'self has incoming edge %s, other has %s' % (selfEdgeName, otherEdgeName)
                pass
        
        selfOutgoingEdges = self.getEdges(self.outgoingEdgeFilter())
        otherOutgoingEdges = other.getEdges(other.outgoingEdgeFilter())

        if not len(selfOutgoingEdges) == len(otherOutgoingEdges):
            yield 'self has %s outgoing edges, other has %s' % (len(selfOutgoingEdges),
                                                                len(otherOutgoingEdges))
        else:
            for selfEdge, otherEdge in zip(sorted(selfOutgoingEdges, Edge.nameComparator),
                                           sorted(otherOutgoingEdges, Edge.nameComparator)):
                selfEdgeName = selfEdge.name()
                otherEdgeName = otherEdge.name()
                if not selfEdgeName == otherEdgeName:
                    yield 'self has outgoing edge %s, other has %s' % (selfEdgeName, otherEdgeName)
                pass
                                                                 
        raise StopIteration

    def isContainer(self):
        return self.__class__.staticIsContainer(self)

    def getKeyForEquivalenceTesting(self):
        return self.id()

    def getEdgeFilter(self):
        theFilter = FilterModule.constructOrFilter()
        theFilter.addFilter(self.incomingEdgeFilter())
        theFilter.addFilter(self.outgoingEdgeFilter())
        return theFilter
    
    
    def numEdges(self, filter=None):
        edges = [x for x in self.getEdges(filter)]
        return len(edges)

    def getEdges(self, aFilter=None):
        
        if aFilter is None:
            aFilter = self.getEdgeFilter()

        return [x for x in aFilter.wrap(self.graph().edges())]

    
    def incomingEdgeFilter(self):
        """
        this can be different than the incoming edge filter
        because there may be other edges (eg from parent to child)
        """
        return FilterModule.ObjectKeyMatchesFilter(
            filter=FilterModule.IdentityFilter(self),
            keyFunction=lambda x: x.convertEntityToNode(x.entities()[1]))
            
        

    def outgoingEdgeFilter(self):
        """
        this can be different than the outgoing edge filter
        because there may be other edges (eg from child to parent)
        """
        return FilterModule.ObjectKeyMatchesFilter(
            filter=FilterModule.IdentityFilter(self),
            keyFunction=lambda x: x.convertEntityToNode(x.entities()[0]))
        

    def predecessors(self):
        theFilter = self.incomingEdgeFilter()
        for edge in self.getEdges(theFilter):
            source = edge.source()
            sourceNode = edge.convertEntityToNode(source)
            yield sourceNode

    def successors(self):
        theFilter = self.outgoingEdgeFilter()
        for edge in self.getEdges(theFilter):
            sink = edge.sink()
            sinkNode = edge.convertEntityToNode(sink)
            yield sinkNode

    def clone(self):
        return Node(id=self.id(), graph=self.graph())

    
    # END class Node
    pass

class Graph (Node):

    ATTRIBUTES = Node.ATTRIBUTES + [
        'nodes',
        'edges'
        ]

    GROUP_EDGES_INCOMING = 1
    GROUP_EDGES_OUTGOING = 2
    GROUP_EDGES_INTERNAL = 3

    FUNCTION_ZERO = currypy.Curry(FilterModule.FUNCTION_IDENTITY,
                                      object=0)
    
    FILTER_NODE_MINIMAL = FilterModule.ObjectFilter(
        valueFunction = FUNCTION_ZERO,
        objectFunction = lambda x: x.numEdges(x.incomingEdgeFilter())
    )

    FILTER_NODE_MAXIMAL = FilterModule.ObjectFilter(
        valueFunction = FUNCTION_ZERO,
        objectFunction = lambda x: x.numEdges(x.outgoingEdgeFilter())
        )

    @staticmethod
    def staticIsContainer(anObject):
        return isinstance(anObject, Graph)

    @staticmethod
    def extractContainer(anObject):
        return anObject


    @staticmethod
    def staticHeadNodeFunction(anObject):
        return anObject.nodes()[0]
    
    def __init__(self, id=None, graph=None):
        Node.__init__(self, id=id, graph=graph)
        
        self.initializeNodes()
        self.initializeEdges()
        return

    
    def __deepcopy__(self, memo):
        if self in memo:
            return memo[self]
        
        result = GraphObject.__deepcopy__(self, memo)
        memo[self] = result
        
        copyNodes = copy.deepcopy(self.nodes(), memo)
        result.nodes(copyNodes)
        
        copyEdges = copy.deepcopy(self.edges(), memo)
        result.edges(copyEdges)
        
        return result
    
    
    def initializeNodes(self):
        # initialize the nodes 
        self.nodes([])
        return
    
    def initializeEdges(self):
        # initialize the edges
        self.edges([])
        return


    def getDifferencesWith(self, other, headNodeFunction=None):
        
        if not type(self) is type(other):
            yield 'difference classes'
            raise StopIteration
        
        if headNodeFunction is None:
            headNodeFunction = self.__class__.staticHeadNodeFunction
            
        # if self.__class__.isContainer(self) and not other.__class__.isContainer(other):
        if self.isContainer() and not other.isContainer():
            yield 'self is container but other is not'
            raise StopIteration
        
        # if not self.__class__.isContainer(self) and other.__class__.isContainer(other):
        if not self.isContainer() and other.isContainer():
            yield 'self is not container but other is'
            raise StopIteration

        if self.isRoot() and not other.isRoot():
            yield 'self is root but other is not'
            raise StopIteration
        
        if not self.isRoot() and other.isRoot():
            yield 'self is not root but other is'
            raise StopIteration

        if not self.isRoot():
            for difference in Node.getDifferencesWith(self, other):
                yield difference
            pass
        
        # check the graph internals
        try:
            assertGraphAutomorphism(self, other,
                                    headNodeFunction,
                                    self.__class__.isContainer,
                                    self.__class__.extractContainer)
        except AssertionError, e:
            yield str(e)
        pass
    
    def __ne__(self, other):
        result = self.__eq__(other)
        if result is NotImplemented:
            return result
        return not result        

    def _containsAll(self, nodes=None, edges=None):
        return set(self.nodes()).issuperset(set(nodes))

    def isRoot(self):
        return (self.graph() is None)

    def copyObjects(self, nodes=None, edges=None, nodeMap=None):
        """
        """
        # initialize the node map if it was not passed in as an argument
        # if it was passed in as an argument, the we use it
        # because the info may be needed outside of this function
        if nodeMap is None:
            nodeMap = {}

        # if the nodes being copied
        if nodes is not None:
            for node in set(nodes):
                self.copyNode(node, nodeMap)
                pass
            pass

        """
        need to process the edges first
        """
        if edges is not None:
            for edge in set(edges):
                self.copyEdge(edge, nodeMap)
                pass
            pass
        # END def copyObjects
        pass

    def getClassForCreateGraph(self):
        return self.__class__

    def createGraph(self, id=None, graphClass=None):
        if graphClass is None:
            graphClass = self.getClassForCreateGraph()
        graph = graphClass(graph=self, id=id)
        self.addNode(graph)
        return graph

    def getClassForCreateNode(self):
        return Node

    def createNode(self, id=None, nodeClass=None):
        if nodeClass is None:
            nodeClass = self.getClassForCreateNode()
        node = nodeClass(graph=self, id=id)
        self.addNode(node)
        return node

    def addNode(self, node):
        node.graph(self)
        self.nodes().append(node)
        pass

    def removeNode(self, node, shouldRemoveEdges=True):
        if not node.graph() is self:
            raise ErrorModule.InvalidValueError(
                "cannot remove node when node is not in this graph")

        if shouldRemoveEdges:
            # remove all edges that going to this node as well
            # need to cache because the underlying c++ iterator
            # gets screwed up if we attempt to remove
            edgesToRemove = [edge for edge in node.getEdges()]
            for edge in edgesToRemove:
                logging.debug("remove node %s remove edge %s" % (node, edge))
                self.removeEdge(edge)

        node.graph(None)
        self.nodes().remove(node)
        return
    
    def copyNode(self, node, nodeMap):
        copiedNode = node.clone()
        self.addNode(copiedNode)
        logging.debug("cloned node %s" % node.id())
        nodeMap[node] = [copiedNode]
        return copiedNode

    def initializeEdgesMapForGrouping(self):
        edgesMap = {
            Graph.GROUP_EDGES_INCOMING:[],
            Graph.GROUP_EDGES_OUTGOING:[],
            Graph.GROUP_EDGES_INTERNAL:[]
            }
        return edgesMap

    def preprocessForModifyExternalsOfNewGroup(self, graph):
        return graph

    def groupNodes(self, nodes):
        # need to ensures are all in this graph
        if not self._containsAll(nodes):
            logging.debug("self.nodes() >> %s" % self.nodes())
            logging.debug("nodes >> %s" % nodes)

            raise ErrorModule.NodeNotExistError(
                "cannot group nodes because they are not in thie graph")
            
        # need to ensure that the nodes form a convex subgraph
        # code here

        # create a new graph and add to self
        graph = self.createGraph()

        # keep track of the 3 types of edges in this situation
        edgesMap = self.initializeEdgesMapForGrouping()

        # ----------------------
        # BEGIN modifying internals of new group
        # ----------------------

        for node in nodes:
            for edge in node.getEdges():
                edgeType = 0
                sourceNode = edge.convertEntityToNode(edge.source())
                sinkNode = edge.convertEntityToNode(edge.sink())
                if sourceNode in nodes:
                    edgeType = edgeType + Graph.GROUP_EDGES_OUTGOING
                if sinkNode in nodes:
                    edgeType = edgeType + Graph.GROUP_EDGES_INCOMING

                edgesMap[edgeType].append(edge)
                pass
            pass

        # for each node and internal edges, copy to graph
        graph.copyObjects(nodes=nodes, edges=edgesMap[Graph.GROUP_EDGES_INTERNAL])
        for node in nodes:
            logging.debug("groupNodes removing node %s" % node)
            self.removeNode(node)
            pass

        # ----------------------
        # BEGIN modifying externals of new group
        # ----------------------
        
        # need to do this because function module
        # makes a distinction between function and functionassignment
        graph = self.preprocessForModifyExternalsOfNewGroup(graph)

        replaceMap = {}
        for node in nodes:
            replaceMap[node] = [graph]
            pass

        # for each external edge, copy and remove original
        # some subclasses (eg function)  automatically create certain edges
        # when nodes are created in a graph
        # so we do not want to copy those edges, which would result
        # in duplicate edges.  instead, we query each class
        # to provide a filter for such edges
        edgesInfo = [
            (Graph.GROUP_EDGES_INCOMING, 
             lambda x: x.convertEntityToNode(x.source()),
             self.getFilterForCopyingIncomingEdgesForGrouping()),
            (Graph.GROUP_EDGES_OUTGOING, 
             lambda x: x.convertEntityToNode(x.sink()),
             self.getFilterForCopyingOutgoingEdgesForGrouping())
            ]
        for edgeType, objectFunction, edgeFilter in edgesInfo:
            # processedNodes keeps track of possibility of an external
            # node having multipe edges into the new group, e.g
            # group(4,5) with edges (2,4) and (2,5)
            processedNodes = set([])
            logging.debug("copying edges for type %s" % edgeType)
            for edge in edgeFilter.wrap(edgesMap[edgeType]):
                externalNode = objectFunction(edge)
                if externalNode in processedNodes:
                    continue
                copiedEdges = self.copyEdge(edge, replaceMap)
                processedNodes.add(externalNode)
                # no need to remove edges here 
                # because they were already removed by copyObjects
                pass
            pass

        # END def groupNodes
        return graph

    def getFilterForCopyingIncomingEdgesForGrouping(self):
        return FilterModule.TRUE_FILTER

    def getFilterForCopyingOutgoingEdgesForGrouping(self):
        return FilterModule.TRUE_FILTER

    def ungroupNode(self, aGraph):
        if not isinstance(aGraph, Graph):
            raise ErrorModule.InvalidValueError("cannot ungroup atomic nodes")

        # get the graph's minimal vertices
        minimalNodes = [node for node in Graph.FILTER_NODE_MINIMAL.wrap(aGraph.nodes())]
        # get the graph's maximal vertices
        maximalNodes = [node for node in Graph.FILTER_NODE_MAXIMAL.wrap(aGraph.nodes())]
        
        nodeMap = {}

        # move all objects (nodes and edges) from graph
        self.copyObjects(nodes=aGraph.nodes(), edges=aGraph.edges(), nodeMap=nodeMap)

        theList = []
        for node in maximalNodes:
            theList = theList + nodeMap[node]
        maximalNodes = theList

        theList = []
        for node in minimalNodes:
            theList = theList + nodeMap[node]
        minimalNodes = theList

        # a list of tuples, each of which is
        # 1. the nodes to replace graph in the copied edges
        # 2. the filter for retrieving the edges to be copied
        copyInfo = [
            (minimalNodes, aGraph.incomingEdgeFilter()),
            (maximalNodes, aGraph.outgoingEdgeFilter())
            ]

        for replaceNodes, edgeFilter in copyInfo:
            replaceMap = {
                aGraph:replaceNodes
                }
            #replaceMap = {}
            #replaceMap.update(nodeMap)
            #replaceMap[graph] = replaceNodes
            for edge in aGraph.getEdges(edgeFilter):
                copiedEdges = self.copyEdge(edge, replaceMap)
                # no need to remove the edge here
                # because we'll be remove the edges
                # when we remove the node
                pass
            pass
        
        self.removeNode(aGraph)
        
        # END def ungroupNodes
        pass

    def getReplaceNodes(self, node, replaceMap):
        nodes = set([node])
        if replaceMap.has_key(node):
            nodes = set(replaceMap[node])
            pass
        return nodes

    def copyEdgeSource(self, source, replaceNodes):
        for replaceNode in replaceNodes:
            yield replaceNode
            pass
        pass

    def copyEdgeSink(self, sink, replaceNodes):
        for replaceNode in replaceNodes:
            yield replaceNode
            pass
        pass

    def copyEdge(self, edge, replaceMap):
        """
        need to update so that this works for edges with more than 2 endpoints
        """
        source = edge.source()
        sourceNode = edge.convertEntityToNode(source)
        sourceNodes = self.getReplaceNodes(sourceNode, replaceMap)

        sink = edge.sink()
        sinkNode = edge.convertEntityToNode(sink)
        sinkNodes = self.getReplaceNodes(sinkNode, replaceMap)

        logging.debug("copying edge %s to edges(%s,%s)" %
                      (edge, sourceNodes, sinkNodes))
        edges = []
        for newSource in self.copyEdgeSource(source, sourceNodes):
            for newSink in self.copyEdgeSink(sink, sinkNodes):
                # should call edge.clone() instead?
                copiedEdge = self.createEdge([newSource, newSink])
                logging.debug("copied to new edge %s" % copiedEdge)
                edges.append(copiedEdge)
                pass
            pass

        return edges

    def getClassForCreateEdge(self):
        return BinaryEdge

    def validateNodesForCreateEdge(self, nodes):
        pass

    def createEdge(self, nodes):
        self.validateNodesForCreateEdge(nodes)
        edgeClass = self.getClassForCreateEdge()
        edge = edgeClass(self, nodes)
        self.addEdge(edge)
        return edge

    def addEdge(self, edge):
        edge.graph(self)
        self.edges().append(edge)
        pass

    def removeEdge(self, edge):
        if not edge.graph() is self:
            raise ErrorModule.InvalidValueError(
                "cannot remove edge when edge is not in this graph")
        edge.graph(None)
        self.edges().remove(edge)
        pass

    # END class Graph
    pass

class Edge (GraphObject):

    ATTRIBUTES = GraphObject.ATTRIBUTES + [
        'entities'
        ]

    
    @staticmethod 
    def nameComparator(relation1, relation2):
        return cmp(relation1.name(), relation2.name())
        
    
    def __init__(self, graph=None, entities=None, id=None):
        """
        its possible to create an edge with no parent graph
        (just need to pass in "None" as graph)
        but we force the API to explicit about such situations
        """
        GraphObject.__init__(self, id=id, graph=graph)
        self.entities(entities)

        # END def __init__
        pass

    def __eq__(self, other):
        if isinstance(other, Edge):
            return self.entities() == other.entities()
        return NotImplemented

    def __deepcopy__(self, memo):
        
        if self in memo:
            return memo[self]
        
        result = GraphObject.__deepcopy__(self, memo)
        
        entities = self.entities()
        copyEntities = map(memo.get, entities, entities)
        result.entities(copyEntities)
        
        result.name(self.name())
        
        memo[self] = result
        return result
    
    def convertEntityToNode(self, x):
        return x

    # END class Edge

class BinaryEdge (Edge):

    ATTRIBUTES = Edge.ATTRIBUTES + []

    def __init__(self, graph=None, nodes=None, id=None):
        Edge.__init__(self, graph=graph, entities=nodes, id=None)
        return

    def __repr__(self):
        return "%s-%s" % (self.source(), self.sink())

    def source(self, value=None):
        entities = self.entities()
        if value is not None:
            entities[0] = value
        return entities[0]

    def sink(self, value=None):
        entities = self.entities()
        if value is not None:
            entities[1] = value
        return entities[1]

    """
    will also need to implement sourcePort and sinkPort
    """

    # END class Edge
    pass

def assertGraphAutomorphism(expected, actual,
                            headNodeFunction,
                            isContainer=Graph.isContainer,
                            extractContainer=Graph.extractContainer):
    differences = [
        x for x in determineGraphAutomorphismDifferences(
            expected, actual, headNodeFunction, isContainer=isContainer, extractContainer=extractContainer)
    ]

    assert len(differences) is 0
    return True

    
def determineGraphAutomorphismDifferences(expected, actual,
                                          headNodeFunction,
                                          isContainer=Graph.isContainer,
                                          extractContainer=Graph.extractContainer):

    if len(expected.nodes()) is not len(actual.nodes()):
        yield 'self has %s nodes, other has %s' % (len(expected.nodes()),
                                                   len(actual.nodes()))
        
    if len(expected.nodes()) is 0 or len(actual.nodes()) is 0:
        raise StopIteration
    
    expectedHead = headNodeFunction(expected)
    actualHead = headNodeFunction(actual)
    
    nodesToIgnore = {}
    nodesToProcess = {expectedHead:actualHead}
    while len(nodesToProcess):

        expectedNode, actualNode = nodesToProcess.popitem()
        if expectedNode in nodesToIgnore:
            continue
        
        nodesToIgnore[expectedNode] = actualNode
        
        if not expectedNode == actualNode:
            yield 'expected node %s is not the same as actual node %s' % (
                expectedNode, actualNode)

        for expectedEdge, actualEdge in \
                zip(sorted(expectedNode.getEdges(expectedNode.incomingEdgeFilter()),
                           Edge.nameComparator),
                    sorted(actualNode.getEdges(actualNode.incomingEdgeFilter()),
                           Edge.nameComparator)):
            expectedPredecessor = expectedEdge.entities()[0]
            actualPredecessor = actualEdge.entities()[0]
            if expectedPredecessor in nodesToProcess:
                if not nodesToProcess[expectedPredecessor] is actualPredecessor:
                    yield 'error on matching edges %s and %s >> expected node %s maps to actual nodes %s and %s' % (
                        expectedEdge, actualEdge, expectedPredecessor, actualPredecessor, nodesToProcess[expectedPredecessor])
                    raise StopIteration
                continue
            nodesToProcess[expectedPredecessor] = actualPredecessor
            pass
        
        for expectedEdge, actualEdge in \
                zip(sorted(expectedNode.getEdges(expectedNode.outgoingEdgeFilter()),
                           Edge.nameComparator),
                    sorted(actualNode.getEdges(actualNode.outgoingEdgeFilter()),
                           Edge.nameComparator)):
            expectedSuccessor = expectedEdge.entities()[1]
            actualSuccessor = actualEdge.entities()[1]
            if expectedSuccessor in nodesToProcess:
                if not nodesToProcess[expectedSuccessor] is actualSuccessor:
                    yield 'error on matching edges %s and %s >> expected node %s maps to actual nodes %s and %s' % (
                        expectedEdge, actualEdge, expectedSuccessor, actualSuccessor, nodesToProcess[expectedSuccessor])
                    raise StopIteration
                continue
            nodesToProcess[expectedSuccessor] = actualSuccessor
            pass
        
        pass
    
    raise StopIteration



class GraphStructureFilter(FilterModule.CompositeFilter):

    def __init__(self):

        FilterModule.CompositeFilter.__init__(
            self,
            FilterModule.AndAccumulator())

        self._initializeGraphTypeFilter()

        self._rawFilters = []
        
        return

    def _initializeGraphTypeFilter(self):
        self._filters.append(
            FilterModule.ObjectFilter(
                valueFunction = FilterModule.FUNCTION_TRUE,
                objectFunction = lambda x: isinstance(x, Graph)
            )
        )

        self._filters[-1].name('isinstance(self, Graph)')
        
        return

    
    def addFilter(self, filter):
        """
        expects filters that match nodes
        """
        theWrappedFilter = GraphNodeWrapperFilter(filter)

        FilterModule.CompositeFilter.addFilter(self, theWrappedFilter)
        self._rawFilters.append(filter)
        return


    # END class GraphStructureFilter
    pass

class GraphNodeWrapperFilter(FilterModule.ObjectFilter):

    @staticmethod
    def match(object, filterHolder=None):
        if filterHolder is None:
            raise NotImplementedError
        
        return any([filterHolder._filter.matches(y) for y in object.nodes()])
    
    
    def __init__(self, filter):
        FilterModule.ObjectFilter.__init__(
            self,
            valueFunction = FilterModule.FUNCTION_TRUE,
            objectFunction = currypy.Curry(GraphNodeWrapperFilter.match,
                                               filterHolder=self)
        )
        self._filter = filter
        return
    
    # END class GraphNodeWrapperFilter
    pass




from __future__ import with_statement

import os
import sys
import threadpool
import unittest
import logging
import uuid

import pypatterns.filter as FilterModule
import pypatterns.relational as RelationalModule

import pomsets.builder as BuilderModule
import pomsets.command as TaskCommandModule
import pomsets.context as ContextModule
import pomsets.definition as DefinitionModule
import pomsets.parameter as ParameterModule

import pomsets.test_utils as TestDefinitionModule

class TestBuilder(unittest.TestCase):

    def setUp(self):
        self.builder = BuilderModule.Builder()
        return

    def testCreateExecutableObject(self):
        path = ['', 'bin', 'echo']
        executable = self.builder.createExecutableObject(path)

        self.assertTrue(isinstance(executable, TaskCommandModule.Executable))
        self.assertFalse(executable.stageable())
        self.assertEquals(path, executable.path())
        self.assertEquals([], executable.staticArgs())
        return


    def testCreateNewAtomicPomset(self):

        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        self.assertTrue(isinstance(pomsetContext, ContextModule.Context))

        pomset = pomsetContext.pomset()
        self.assertTrue(isinstance(pomset, DefinitionModule.AtomicDefinition))

        self.assertTrue(pomset.functionToExecute() is DefinitionModule.executeTaskInEnvironment)

        executable = pomset.executable()
        self.assertTrue(executable is executableObject)


        self.assertEquals('shell process', pomset.commandBuilderType())

        return


    def testRemoveParameter1(self):
        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        self.assertRaises(ValueError,
                          self.builder.removePomsetParameter,
                          pomset,
                          DefinitionModule.Definition.SYMBOL_INPUT_TEMPORAL)

        self.assertRaises(ValueError,
                          self.builder.removePomsetParameter,
                          pomset,
                          DefinitionModule.Definition.SYMBOL_OUTPUT_TEMPORAL)
        return


    def testRemoveParameter2(self):
        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        # test for regular input value
        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_INPUT,
            }
        parameterName = 'input value'
        parameter = self.builder.addPomsetParameter(
            pomset, parameterName, attributes)

        self.assertTrue(pomset.hasParameter(parameterName))
        self.assertTrue(pomset.getParameter(parameterName) is parameter)

        self.builder.removePomsetParameter(pomset, parameterName)
        
        self.assertFalse(pomset.hasParameter(parameterName))

        return

    def testAddParameterToAtomicPomset1(self):
        """
        test for basic input value (non-file) parameter
        using all default values for the attribute
        """

        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        # test for regular input value
        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_INPUT,
            }
        parameterName = 'input value'
        parameter = self.builder.addPomsetParameter(
            pomset, parameterName, attributes)

        self.assertTrue(pomset.hasParameter(parameterName))
        self.assertTrue(pomset.getParameter(parameterName) is parameter)

        self.assertEquals(False, parameter.optional())
        self.assertEquals(True, parameter.active())
        self.assertEquals(
            True,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISLIST))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISENUM))

        commandlineOptions = parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS)
        self.assertEquals(
            [], commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG])
        self.assertEquals(
            False, commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE])
        self.assertEquals(
            {}, commandlineOptions[ParameterModule.COMMANDLINE_ENUM_MAP])


        return


    def testAddParameterToAtomicPomset2(self):
        """
        test for default input file parameter
        """
        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_INPUT,
            'file':True
            }
        parameterName = 'input file'
        parameter = self.builder.addPomsetParameter(
            pomset, parameterName, attributes)

        self.assertTrue(pomset.hasParameter(parameterName))
        self.assertTrue(pomset.getParameter(parameterName) is parameter)

        self.assertEquals(False, parameter.optional())
        self.assertEquals(True, parameter.active())
        self.assertEquals(
            True,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE))
        self.assertEquals(
            True,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISLIST))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISENUM))

        commandlineOptions = parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS)
        self.assertEquals(
            [], commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG])
        self.assertEquals(
            False, commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE])
        self.assertEquals(
            {}, commandlineOptions[ParameterModule.COMMANDLINE_ENUM_MAP])

        return


    def testAddParameterToAtomicPomset3(self):
        """
        test for default output file parameter
        """

        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_OUTPUT,
            'file':True
            }
        parameterName = 'output file'
        parameter = self.builder.addPomsetParameter(
            pomset, parameterName, attributes)

        self.assertTrue(pomset.hasParameter(parameterName))
        self.assertTrue(pomset.getParameter(parameterName) is parameter)

        self.assertEquals(False, parameter.optional())
        self.assertEquals(True, parameter.active())
        self.assertEquals(
            True,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE))
        self.assertEquals(
            True,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISLIST))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISENUM))

        commandlineOptions = parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS)
        self.assertEquals(
            [], commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG])
        self.assertEquals(
            False, commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE])
        self.assertEquals(
            {}, commandlineOptions[ParameterModule.COMMANDLINE_ENUM_MAP])

        return


    def testAddParameterOrdering(self):
        """
        This verifies that we can create a parameter ordering
        """

        path = ['', 'bin', 'echo']
        executableObject = self.builder.createExecutableObject(path)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_INPUT,
            'file':True
            }
        sourceParameterName = 'input file'
        parameter = self.builder.addPomsetParameter(
            pomset, sourceParameterName, attributes)

        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_OUTPUT,
            'file':True
            }
        targetParameterName = 'output file'
        parameter = self.builder.addPomsetParameter(
            pomset, targetParameterName, attributes)


        parameterOrderings = pomset.parameterOrderingTable()

        self.assertTrue(parameterOrderings.rowCount() is 0)
        self.builder.addParameterOrdering(
            pomset, sourceParameterName, targetParameterName)
        
        self.assertTrue(parameterOrderings.rowCount() is 1)
        rowValue = parameterOrderings.rows()[0].values()
        
        self.assertEquals(sourceParameterName, rowValue['source'])
        self.assertEquals(targetParameterName, rowValue['target'])

        return


    def testCreateNewNestPomset(self):

        pomsetContext = self.builder.createNewNestPomset()
        self.assertTrue(isinstance(pomsetContext, ContextModule.Context))

        pomset = pomsetContext.pomset()
        self.assertTrue(isinstance(pomset, DefinitionModule.NestDefinition))

        return

    def testCreateNewNode(self):
        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()

        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT

        node = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        self.assertTrue(isinstance(node, DefinitionModule.ReferenceDefinition))

        self.assertTrue(node.definitionToReference(),
                        mapDefinition)

        return


    def testCanConnect1(self):
        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT
        reduceDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT_REDUCE

        node1 = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        node2 = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)

        # ensure that cannon connect a parameter to itself
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'input file',
                node1, 'input file'))

        # ensure false if the paraemter does not exist
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'input',
                node2, 'output file'))
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'input file',
                node2, 'output'))

        # ensure cannot connect parameters if they are of the same direction
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'input file',
                node2, 'input files'))

        # ensure that we cannot connect if not the same type
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'temporal input',
                node2, 'input files'))
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'input file',
                node2, 'temporal input'))
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'temporal input',
                node2, 'output file'))
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'output file',
                node2, 'temporal input'))

        # ensure that the target parameter needs to be of direction input
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'output file',
                node2, 'output file'))
        
        # ensure that source parameter needs to be of direction output
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'input file',
                node2, 'input files'))

        # now ensure we can connect the following
        self.assertTrue(
            self.builder.canConnect(
                pomset,
                node1, 'output file',
                node2, 'input files'))
        self.assertTrue(
            self.builder.canConnect(
                pomset,
                node1, 'temporal output',
                node2, 'temporal input'))

        return

    def testCanConnect2(self):
        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT
        reduceDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT_REDUCE

        node1 = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        node2 = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)

        # cannot connect if connection is temporal and already exists
        self.assertTrue(
            self.builder.canConnect(
                pomset,
                node1, 'temporal output',
                node2, 'temporal input'))
        self.builder.connect(
            pomset,
            node1, 'temporal output',
            node2, 'temporal input')
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'temporal output',
                node2, 'temporal input'))

        # cannot connect if connection is data and already exists
        self.assertTrue(
            self.builder.canConnect(
                pomset,
                node1, 'output file',
                node2, 'input files'))
        self.builder.connect(
            pomset,
            node1, 'output file',
            node2, 'input files')
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node1, 'output file',
                node2, 'input files'))
        
        # cannot connect if target parameter is data
        # and already connected to something else
        node3 = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        self.assertFalse(
            self.builder.canConnect(
                pomset,
                node3, 'output file',
                node2, 'input files'))

        return

    def testConnectDataParameters(self):

        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT
        reduceDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT_REDUCE

        sourceNode = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        targetNode = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)

        sourceParameterId = 'output file'
        targetParameterId = 'input files'
        bbParameterName = '%s.%s-%s.%s' % (sourceNode.name(),
                                           sourceParameterId,
                                           targetNode.name(),
                                           targetParameterId)
        self.assertFalse(pomset.hasParameter(bbParameterName))

        # connect the data parameters
        path = self.builder.connect(
            pomset,
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)

        # assert the parameter connection path
        self.assertEquals(len(path), 7)
        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)
        rows = [x for x in 
            pomset.parameterConnectionPathTable().retrieve(
                filter, ['path', 'additional parameters'])]
        self.assertEquals(len(rows), 1)
        row = rows[0]
        self.assertEquals(tuple([path[2],path[4]]),
                          row[0])
        self.assertEquals(tuple([path[3]]), row[1])

        # assert two additional entries in parameter connections
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(0, len(connections))

        # verify that each of the individual connections
        # are in the table
        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            pomset, bbParameterName)
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(1, len(connections))
        filter = pomset.constructParameterConnectionFilter(
            pomset, bbParameterName,
            targetNode, targetParameterId)
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(1, len(connections))
        
        # assert an additional internal blackboard parameter
        self.assertTrue(pomset.hasParameter(bbParameterName))



        self.builder.disconnect(pomset,
                                sourceNode, sourceParameterId,
                                targetNode, targetParameterId)

        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)
        rows = [x for x in 
            pomset.parameterConnectionPathTable().retrieve(
                filter, ['path', 'additional parameters'])]
        self.assertEquals(len(rows), 0)

        # verify that each of the individual connections
        # are in the table
        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            pomset, bbParameterName)
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(0, len(connections))
        filter = pomset.constructParameterConnectionFilter(
            pomset, bbParameterName,
            targetNode, targetParameterId)
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(0, len(connections))


        self.assertFalse(pomset.hasParameter(bbParameterName))
        return


    def testConnectTemporalParameters(self):

        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT
        reduceDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT_REDUCE

        sourceNode = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        targetNode = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)

        sourceParameterId = 'temporal output'
        targetParameterId = 'temporal input'
        parameters = pomset.getParametersByFilter(FilterModule.TRUE_FILTER)
        numParameters = len(parameters)
        path = self.builder.connect(
            pomset,
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)

        # assert parameter connection path
        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)
        rows = [x for x in 
            pomset.parameterConnectionPathTable().retrieve(
                filter, ['path', 'additional parameters'])]
        self.assertEquals(len(rows), 1)
        row = rows[0]
        self.assertEquals(1, len(row[0]))
        self.assertTrue(path[2] is row[0][0])
        self.assertEquals(0, len(row[1]))

        # assert no additional parameters
        parameters = pomset.getParametersByFilter(FilterModule.TRUE_FILTER)
        self.assertEquals(numParameters, len(parameters))
        # assert the parameter connection
        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(1, len(connections))


        self.builder.disconnect(
            pomset,
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)
        parameters = pomset.getParametersByFilter(FilterModule.TRUE_FILTER)
        self.assertEquals(numParameters, len(parameters))
        filter = pomset.constructParameterConnectionFilter(
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)
        connections = RelationalModule.Table.reduceRetrieve(
            pomset.parameterConnectionsTable(),
            filter, ['parameter connection'])
        self.assertEquals(0, len(connections))
        

        return


    def testRemoveNode(self):

        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT
        reduceDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT_REDUCE

        sourceDataNode = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        sourceTemporalNode = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        nodeToRemove = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        targetDataNode = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)
        targetTemporalNode = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)

        self.assertTrue(5, len(pomset.nodes()))

        path = self.builder.connect(
            pomset,
            sourceDataNode, 'output file',
            nodeToRemove, 'input file')
        path = self.builder.connect(
            pomset,
            nodeToRemove, 'output file',
            targetDataNode, 'input files')
        path = self.builder.connect(
            pomset,
            sourceTemporalNode, 'temporal output',
            nodeToRemove, 'temporal input')
        path = self.builder.connect(
            pomset,
            nodeToRemove, 'temporal output',
            targetTemporalNode, 'temporal input')


        self.builder.removeNode(pomset, nodeToRemove)
        
        self.assertEquals(0, pomset.parameterConnectionPathTable().rowCount())
        self.assertEquals(0, pomset.parameterConnectionsTable().rowCount())

        self.assertFalse(nodeToRemove in pomset.nodes())

        return


    def testParameterBinding1(self):
        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT

        node = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)

        # no connection means parameters are modified directly
        parameterId = 'input file'
        boundValue = ['foo']
        self.builder.bindParameterValue(node, parameterId,
                                        boundValue)

        self.assertEquals(boundValue,
                          node.getParameterBinding(parameterId))

        # no connection means parameters are modified directly
        parameterId = 'output file'
        boundValue = ['bar']
        self.builder.bindParameterValue(node, parameterId,
                                        boundValue)
        self.assertEquals(boundValue,
                          node.getParameterBinding(parameterId))
        return



    def testParameterBinding2(self):

        pomsetContext = self.builder.createNewNestPomset()
        pomset = pomsetContext.pomset()
        
        mapDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT
        reduceDefinition = TestDefinitionModule.DEFINITION_WORDCOUNT_REDUCE

        sourceNode = self.builder.createNewNode(
            pomset, definitionToReference=mapDefinition)
        targetNode = self.builder.createNewNode(
            pomset, definitionToReference=reduceDefinition)

        sourceParameterId = 'output file'
        targetParameterId = 'input files'

        # connect the data parameters
        path = self.builder.connect(
            pomset,
            sourceNode, sourceParameterId,
            targetNode, targetParameterId)

        # setting the binding on an output file
        # modifies that parameter directly
        boundValue = ['foo']
        self.builder.bindParameterValue(sourceNode, sourceParameterId,
                                        boundValue)
        self.assertEquals(boundValue,
                          sourceNode.getParameterBinding(sourceParameterId))


        # setting the binding on an input file
        # modifies that the incoming blackboard parameter
        boundValue = ['bar']
        self.builder.bindParameterValue(targetNode, targetParameterId,
                                        boundValue)
        bbParameterName = '%s.%s-%s.%s' % (sourceNode.name(),
                                           sourceParameterId,
                                           targetNode.name(),
                                           targetParameterId)
        self.assertEquals(boundValue,
                          pomset.getParameterBinding(bbParameterName))
        self.assertFalse(targetNode.hasParameterBinding(targetParameterId))

        return

    # END class TestBuilder
    pass



import os
import sys
import unittest
import logging


import pomsets.error as ErrorModule
import pomsets.graph as GraphModule

class TestCase(unittest.TestCase):

    def createGroupedGraph1(self):
        nodeMap = {}
        
        id =0
        aGraph = GraphModule.Graph(id=id)
        nodeMap[id] = aGraph

        for id in [1,2,3,9,10]:
            node = aGraph.createNode(id=id)
            nodeMap[id] = node
            pass

        id=11
        aGraph = aGraph.createGraph(id=id)
        nodeMap[id] = aGraph

        for id in [4,5,6,7,8]:
            node = aGraph.createNode(id=id)
            nodeMap[id] = node
            pass

        edgesInfo = [
            (0, 1, 11),
            (0, 2, 11),
            (0, 3, 11),
            (0, 11, 9),
            (0, 11, 10),
            (11, 4, 5),
            (11, 4, 6),
            (11, 5, 8),
            (11, 6, 8),
            ]

        for graphId, source, target in edgesInfo:
            nodes = [nodeMap[source], nodeMap[target]]
            nodeMap[graphId].createEdge(nodes)
            pass

        return nodeMap[0], nodeMap

    def createUngroupedGraph1(self):
        def idGenerator():
            id = -1
            while 1:
                id = id + 1
                yield id
                pass
            pass

        idFunction = idGenerator().next

        nodeMap = {}
        createFunction = GraphModule.Graph
        for idx in range(1,12):
            id = idFunction()
            if idx is 2:
                createFunction = nodeMap[0].createNode

            node = createFunction(id=id)
            nodeMap[id] = node
            pass

        edgesInfo = [
            (0,1,4),
            (0,1,7),
            (0,2,4),
            (0,3,4),
            (0,4,5),
            (0,4,6),
            (0,5,8),
            (0,6,8),
            (0,8,9),
            (0,8,10)
            ]
        for graphId, source, target in edgesInfo:
            nodes = [nodeMap[source], nodeMap[target]]
            nodeMap[graphId].createEdge(nodes)
            pass

        return nodeMap[0], nodeMap

    def createUngroupedGraph2(self):
        def idGenerator():
            id = -1
            while 1:
                id = id + 1
                yield id
                pass
            pass

        idFunction = idGenerator().next

        nodeMap = {}
        createFunction = GraphModule.Graph
        for idx in range(1,12):
            id = idFunction()
            if idx is 2:
                createFunction = nodeMap[0].createNode

            node = createFunction(id=id)
            nodeMap[id] = node
            pass

        edgesInfo = [
            (0,1,4),
            (0,1,7),
            (0,2,4),
            (0,2,7),
            (0,3,4),
            (0,3,7),
            (0,4,5),
            (0,4,6),
            (0,5,8),
            (0,6,8),
            (0,7,9),
            (0,7,10),
            (0,8,9),
            (0,8,10),
            ]
        for graphId, source, target in edgesInfo:
            nodes = [nodeMap[source], nodeMap[target]]
            nodeMap[graphId].createEdge(nodes)
            pass

        return nodeMap[0], nodeMap


    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testDifferentGraph(self):
        expectedGraph, expectedNodeMap = self.createUngroupedGraph2()
        badGraph, badNodeMap = self.createUngroupedGraph1()
        try:
            GraphModule.assertGraphAutomorphism(expectedGraph, badGraph,
                                                lambda x: x.nodes()[0])
            raise ErrorModule.InvalidValueError("expected to fail")
        except AssertionError:
            pass
        return
    
    
    # TODO:
    # will need to fix this test
    def _testUngroupNode(self):
        expectedGraph, expectedNodeMap = self.createUngroupedGraph2()
        actualGraph, actualNodeMap = self.createGroupedGraph1()
        badGraph, badNodeMap = self.createUngroupedGraph1()

        actualGraph.ungroupNode(actualNodeMap[11])

        logging.debug("testUngroupNode expectedNodes >> %s" % expectedGraph.nodes())
        logging.debug("testUngroupNode actualNodes >> %s" % actualGraph.nodes())

        logging.debug("testUngroupNode expectedEdges >> %s" % expectedGraph.edges())
        logging.debug("testUngroupNode actualNodes >> %s" % actualGraph.edges())

        GraphModule.assertGraphAutomorphism(expectedGraph, actualGraph,
                                            lambda x: x.nodes()[0])

        pass

    
    # TODO:
    # will need to fix this test
    def _testGroupNodes(self):
        
        actualGraph, actualNodeMap = self.createUngroupedGraph1()
        expectedGraph, expectedNodeMap = self.createGroupedGraph1()

        nodesToGroup = []
        for id in [4,5,6,7,8]:
            node = actualNodeMap[id]
            nodesToGroup.append(node)
            logging.debug("appended node %s for id %s" % (node.id(), id))
        group = actualGraph.groupNodes(nodesToGroup)
        group.id(11)

        GraphModule.assertGraphAutomorphism(expectedGraph, actualGraph,
                                            lambda x: x.nodes()[-1])
        pass

    
    def testEdgeFilters(self):
        theGraph = GraphModule.Graph(id='graph')
        theNode1 = theGraph.createNode(id='node1')
        theNode2 = theGraph.createNode(id='node2')
        theEdge = theGraph.createEdge([theNode1, theNode2])

        assert theNode1.outgoingEdgeFilter().matches(theEdge)
        assert not theNode1.incomingEdgeFilter().matches(theEdge)
        
        theEdges = theNode1.getEdges()
        assert theEdge in theEdges
        
        assert theNode2.incomingEdgeFilter().matches(theEdge)
        assert not theNode2.outgoingEdgeFilter().matches(theEdge)
        
        theEdges = theNode2.getEdges()
        assert theEdge in theEdges
        
        return
    
    # END class TestCase


def main():
    util.configLogging()
    suite = unittest.makeSuite(TestCase,'test')
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()





import pomsets.graph as GraphModule
import pomsets.resource as ResourceModule

PORT_TYPE_DATA = 'data'
PORT_TYPE_BLACKBOARD = 'blackboard'
PORT_TYPE_TEMPORAL = 'temporal'
PORT_DIRECTION_INPUT = 'input'
PORT_DIRECTION_OUTPUT = 'output'
PORT_DIRECTION_INTERNAL = 'internal'

PORT_ATTRIBUTE_COMMANDLINE = 'isCommandline'
PORT_ATTRIBUTE_COMMANDLINE_OPTIONS = 'commandlineOptions'
PORT_ATTRIBUTE_PARAMETERSWEEP = 'isParameterSweep'
PORT_ATTRIBUTE_ISSIDEEFFECT = 'isSideEffect'
PORT_ATTRIBUTE_ISINPUTFILE = 'isInputFile'
PORT_ATTRIBUTE_ISLIST = 'isList'
PORT_ATTRIBUTE_ISENUM = 'isEnum'
PORT_ATTRIBUTE_ISOPTIONAL = 'isOptional'
PORT_ATTRIBUTE_DESCRIPTION = 'description'

# these are actually dynamic parameter attributes
# PORT_ATTRIBUTE_STAGINGREQUIRED = 'stagingRequired'
# PORT_ATTRIBUTE_PSGROUP = 'parameter sweep group'

COMMANDLINE_ENUM_MAP = 'enum map'
# what the flag to the commandline argument should be
COMMANDLINE_PREFIX_FLAG = 'prefix flag'
# if the parameter has a list of values,
# True if the flag should be prepended for each value,
# False (default) if the flag should be prepended just once
COMMANDLINE_PREFIX_FLAG_DISTRIBUTE = 'distribute prefix flag'
# TODO:
# need an attribute as to whether the value should be quoted
# need an attribute as to whether the quote should be distributed
# need an attribute to specify what the start/end quote chars should be



def setAttributes(parameter, attributes):
    for key, value in attributes.iteritems():
        parameter.setAttribute(key, value)
    return


class Parameter(ResourceModule.Struct):

    ATTRIBUTES = [
        'id', 'name', 'definition', 'attributes', 'description'
    ]

    def __init__(self, id=None, name=None,
                 possibleValues=None, defaultValue=None):

        ResourceModule.Struct.__init__(self)

        if id is None:
            id = ResourceModule.Resource.ID_GENERATOR()
        self.id(id)
        
        if name is None:
            name = ''
        self.name(name)

        self.possibleValues(possibleValues)

        if defaultValue is not None:
            self.defaultValue(defaultValue)

        self.attributes({})

        pass


    def __str__(self):
        return self.id()

    def __repr__(self):
        return self.id()

    def __eq__(self, other):
        if type(self) == type(other):
            return self.id() == other.id() and \
                   self.function().id() == other.function().id()
        return NotImplemented

    def getAttribute(self, attributeName, defaultValue=False):
        return self.attributes().get(attributeName, defaultValue)

    def setAttribute(self, attributeName, value):
        self.attributes()[attributeName] = value
        return


    def possibleValues(self, possibleValues=None):
        """
	will need to implement to deal with the situation where
	the new set of possible values does not allow for 
	an already set value
	"""
        if possibleValues is not None:
            self.__possibleValues = possibleValues
            pass
        try:
            self.__possibleValues
        except AttributeError:
            #if possible values undefined, default to unrestricted
            self.__possibleValues = UnrestrictedPossibleValues()
            pass
        return self.__possibleValues


    def defaultValue(self, defaultValue=None):
        if defaultValue is not None:
            #validate that the default value is permitted
            possibleValues = self.possibleValues()
            try:
                possibleValues.validateValue(defaultValue)
            except:
                raise InvalidValueError("default value not permitted by specified set of possible values")
            self.__defaultValue = defaultValue
            pass
        
        try:
            self.__defaultValue
        except AttributeError:
            #if no default defined, default to none
            return None
        return self.__defaultValue

    pass



class ExposedParameter(Parameter):
    """
    the externally exposed parameters of a function are defined here
    """

    ATTRIBUTES = Parameter.ATTRIBUTES + [
        'portDirection',
        'portType',
        'optional',
        'active'
    ]

    def __init__(self, optional=True, active=False, 
                 portDirection=None, portType=None, **kwds):
        Parameter.__init__(self, **kwds)
        self.optional(optional)
        self.active(active)
        self.portDirection(portDirection)
        self.portType(portType)
        return

    def __eq__(self, other):
        differences = [x for x in self.getDifferencesWith(other)]
        return len(differences) is not 0
    
    def getDifferencesWith(self, other):
        if not other:
            yield 'other is None'
            raise StopIteration
        if not self.__class__ == other.__class__:
            yield 'other is of class %s' % other.__class__
            raise StopIteration
        if not self.id() == other.id():
            yield 'id'
        if not self.optional() == other.optional():
            yield 'optional'
        if not self.active() == other.active():
            yield 'active'
        if not self.portDirection() == other.portDirection():
            yield 'portDirection'
        if not self.portType() == other.portType():
            yield 'portType'
        raise StopIteration
    
    
    # END class ExposedParameter
    pass


class BlackboardParameter(ExposedParameter):

    ATTRIBUTES = ExposedParameter.ATTRIBUTES + []

    def __init__(self, id=None, **kwds):
        ExposedParameter.__init__(self, id=id, 
                                  portDirection=PORT_DIRECTION_INTERNAL,
                                  portType=PORT_TYPE_BLACKBOARD)
        return

    # END class BlackboardParameter
    pass


class PortParameter(ExposedParameter):

    ATTRIBUTES = ExposedParameter.ATTRIBUTES + []

    def __init__(self, **kwds):
        ExposedParameter.__init__(self, **kwds)
        return

    # END class PortParameter
    pass

class DataParameter(PortParameter):

    ATTRIBUTES = PortParameter.ATTRIBUTES + []

    def __init__(self, **kwds):
        PortParameter.__init__(self, 
                               portType=PORT_TYPE_DATA,
                               **kwds)

        return

    # END class DataParameter
    pass



class InputTemporalParameter(PortParameter):

    ATTRIBUTES = PortParameter.ATTRIBUTES + []

    def __init__(self, id=None):
        PortParameter.__init__(self, id=id, 
                               portDirection=PORT_DIRECTION_INPUT,
                               portType=PORT_TYPE_TEMPORAL,
                               optional=True,
                               active=True)

        return

    # END class InputTemporalParameter
    pass




class OutputTemporalParameter(PortParameter):

    ATTRIBUTES = PortParameter.ATTRIBUTES + []

    def __init__(self, id=None):
        PortParameter.__init__(self, id=id, 
                               portType=PORT_TYPE_TEMPORAL,
                               portDirection=PORT_DIRECTION_OUTPUT,
                               optional=True,
                               active=True)
        # END def __init__
        pass

    # END class OutputTemporalParameter
    pass



class PossibleValues(ResourceModule.Resource):
    def __init__(self):
        pass

    def __iter__(self):
        raise NotImplementedError

    def validateValue(self, value):
        raise NotImplementedError

    pass

class RangePossibleValues(PossibleValues):
    def __init__(self):
        pass

    def __iter__(self):
        raise NotImplementedError

    def validateValue(self, value):
        """ x >= min && x <= max && x-min/step = n
	"""
        if value < min:
            raise InvalidValueError
        if value > max:
            raise InvalidValueError
        val = (value-min)/step
        if val != int(val):
            raise InvalidValueError
        return

    def min(self, min=0):
        raise NotImplementedError

    def max(self, max=-1):
        raise NotImplementedError

    def step(self, step=1):
        raise NotImplementedError

    pass

class UnrestrictedPossibleValues(PossibleValues):
    """ a single value, unrestricted in its contents """
    def __init__(self):
        pass

    def __iter__(self):
        raise NotImplementedError

    def validateValue(self, value):
        return

    pass

class EnumerationPossibleValues(PossibleValues):
    def __init__(self, values=None):
        PossibleValues.__init__(self, values)
        pass

    def validateValue(self, value):
        for enumValue in self:
            if value == enumValue:
                return
            pass
        raise InvalidValueError

    def __iter__(self):
        raise NotImplementedError

    def addValue(self):
        raise NotImplementedError

    def removeValue(self):
        raise NotImplementedError

    pass

class DefaultValue(ResourceModule.ResourceReference):
    def __init__(self, possibleValues=None):
        self.__possibleValues = possibleValues

    def value(self, value=None):
        if value is not None:
            #if it is not valid
            #raise InvalidValueError

            #set value
            pass
        return self.value

    pass



class ParameterConnection(GraphModule.Edge):

    ATTRIBUTES = GraphModule.Edge.ATTRIBUTES + [
        'sourceNode',
        'targetNode',
        'sourceParameter',
        'targetParameter'
    ]

    def __init__(self, definition=None, nodes=None):
        GraphModule.Edge.__init__(self, graph=definition, entities=nodes)

        return

    def setReferences(self,
                      sourceNode, sourceParameter,
                      targetNode, targetParameter):

        """
	This should only be called in conjunction with
	the constructor
	"""

        self.sourceNode(sourceNode)
        self.targetNode(targetNode)
        self.sourceParameter(sourceParameter)
        self.targetParameter(targetParameter)
        return


    def source(self):
        return (self.sourceNode(), self.sourceParameter())

    def target(self):
        return (self.targetNode(), self.targetParameter())

    def sink(self):
        return self.target()

    def entities(self, value=None):
        return [self.source(), self.target()]

    def convertEntityToNode(self, entity):
        return entity[0]

    pass




from setuptools import setup, find_packages

setup(
    name='pomsets-core',
    version='1.0.2',
    packages=find_packages(
        'src',exclude=["*.test", "*.test.*", "test.*", "test",
                       "*.utils", "*.utils.*", "utils.*", "utils",]),
    package_dir={'':'src'},
    test_suite="test",
    tests_require=[
        "euca2ools>=1.1"
        ],
    install_requires = [
        'cloudpool>=0.1.2',
        'pypatterns>=0.1.1',
        'Reaction>=0.2',
        'simplejson>=2.0.9'
        ],

    # metadata for upload to PyPI
    author = "michael j pan",
    author_email = "mikepan@gmail.com",
    description = "workflow management for the cloud",
    license = "GPL (personal use) or commercial",
    keywords = "workflow cloud hadoop parameter sweep",
    url = "http://pomsets.org",
    long_description = "pomsets-core implements the core functionality for workflow management in the cloud.",
    platforms = ["All"]

)



import os
import unittest

import pomsets.context as ContextModule
import pomsets.definition as DefinitionModule
import pomsets.hadoop as HadoopModule
import pomsets.parameter as ParameterModule


if not os.getenv('HADOOP_HOME'):
    os.environ['HADOOP_HOME'] = os.path.sep.join(['', 'hadoop'])


class TestBuilder(unittest.TestCase):


    def setUp(self):
        self.builder = HadoopModule.Builder()
        return

    def testCreateDefaultExecutable(self):

        jarFile = 'myJarFile'
        jarClass = 'myJarClass'
        executableObject = self.builder.createExecutableObject(jarFile, jarClass)

        self.assertTrue(isinstance(executableObject, HadoopModule.JarExecutable))
        self.assertFalse(executableObject.stageable())
        self.assertEquals([HadoopModule.getExecutablePath()],
                          executableObject.path())
        self.assertEquals([jarFile],
                          executableObject.jarFile())
        self.assertEquals([jarClass],
                          executableObject.jarClass())

        return


    def testCreateStreamingExecutable(self):

        executableObject = self.builder.createStreamingExecutableObject()

        self.assertTrue(isinstance(executableObject, HadoopModule.JarExecutable))
        self.assertFalse(executableObject.stageable())
        self.assertEquals([HadoopModule.getExecutablePath()],
                          executableObject.path())
        self.assertEquals([HadoopModule.getStreamingJar()],
                          executableObject.jarFile())
        self.assertEquals([],
                          executableObject.jarClass())

        return


    def testCreatePipesExecutable(self):

        path = os.path.sep.join(['', 'path', 'to', 'pipes'])
        executableObject = self.builder.createPipesExecutableObject(path)
        
        self.assertTrue(isinstance(executableObject, HadoopModule.PipesExecutable))
        self.assertFalse(executableObject.stageable())
        self.assertEquals([HadoopModule.getExecutablePath()],
                          executableObject.path())
        self.assertEquals([path],
                          executableObject.pipesFile())

        return


    def testCreateDefinition(self):

        jarFile = 'myJarFile'
        jarClass = 'myJarClass'
        executableObject = self.builder.createExecutableObject(jarFile, jarClass)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        self.assertTrue(isinstance(pomsetContext, ContextModule.Context))

        pomset = pomsetContext.pomset()
        self.assertTrue(isinstance(pomset, DefinitionModule.AtomicDefinition))

        self.assertTrue(pomset.functionToExecute() is DefinitionModule.executeTaskInEnvironment)

        executable = pomset.executable()
        self.assertTrue(executable is executableObject)

        self.assertEquals('shell process', pomset.commandBuilderType())

        return


    def testAddParameterToAtomicPomset1(self):
        """
        test for basic input value (non-file) parameter
        using all default values for the attribute
        """

        jarFile = 'myJarFile'
        jarClass = 'myJarClass'
        executableObject = self.builder.createExecutableObject(jarFile, jarClass)

        pomsetContext = self.builder.createNewAtomicPomset(
            executableObject=executableObject)
        pomset = pomsetContext.pomset()

        # test for regular input value
        attributes = {
            'direction':ParameterModule.PORT_DIRECTION_INPUT,
            }
        parameterName = 'input value'
        parameter = self.builder.addPomsetParameter(
            pomset, parameterName, attributes)

        self.assertTrue(pomset.hasParameter(parameterName))
        self.assertTrue(pomset.getParameter(parameterName) is parameter)

        self.assertEquals(False, parameter.optional())
        self.assertEquals(True, parameter.active())
        self.assertEquals(
            True,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_COMMANDLINE))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISLIST))
        self.assertEquals(
            False,
            parameter.getAttribute(ParameterModule.PORT_ATTRIBUTE_ISENUM))

        commandlineOptions = parameter.getAttribute(
            ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS)
        self.assertEquals(
            [], commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG])
        self.assertEquals(
            False, commandlineOptions[ParameterModule.COMMANDLINE_PREFIX_FLAG_DISTRIBUTE])
        self.assertEquals(
            {}, commandlineOptions[ParameterModule.COMMANDLINE_ENUM_MAP])


        return


    # END class TestBuilder
    pass




"""
import unittest

import TestExecuteHadoop


def additional_tests():
    suite = unittest.TestSuite()

    suite.addTest(unittest.makeSuite(TestExecuteHadoop.TestHadoop1, 'test'))
    suite.addTest(unittest.makeSuite(TestExecuteHadoop.TestHadoopStreaming1, 'test'))
    return suite

"""



import copy
import os
import sys
import unittest
import logging

import StringIO

#import utils
#utils.setPythonPath()


import threadpool

import pypatterns.command as CommandPatternModule

import pomsets.automaton as AutomatonModule
import pypatterns.filter as FilterModule

import cloudpool as CloudModule

import pomsets.command as TaskCommandModule
import pomsets.task as TaskModule

import pomsets.test_utils as GeneratePomsetsModule



class PrintObject(object):
    
    def __init__(self):
        automaton = AutomatonModule.Automaton()
        automaton.setThreadPool(0, CloudModule.Pool(1))
        automaton.commandManager(CommandPatternModule.CommandManager())
        self.automaton = automaton
        return

    def createCommandBuilder(self):
        commandBuilder = TaskCommandModule.CommandBuilder(
            TaskCommandModule.buildCommandFunction_commandlineArgs
        )
        return commandBuilder

    def createExecuteEnvironment(self):
        return ShellModule.LocalShell()

    
    def createTask(self, definition):

        compositeTask = TaskModule.CompositeTask()
        compositeTask.definition(definition)
        taskGenerator = TaskModule.NestTaskGenerator()
        compositeTask.taskGenerator(taskGenerator)
        return compositeTask
    
    
    def getRequestContext(self, task=None, executeEnvironment=None):
        
        automaton = self.automaton
        commandBuilder = self.createCommandBuilder()

        if executeEnvironment is None:
            executeEnvironment = self.createExecuteEnvironment()
            
        requestContext = {
            'task':task,
            'command builder':commandBuilder,
            'execute environment':executeEnvironment
        }
        return requestContext

    
    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = TaskCommandModule.PrintTaskCommand()
        self.env.outputStream(io)
        return self.env

    def executeTask(self, task, 
                    executeEnvironment=None):
            
        automaton = self.automaton
        successCallback = automaton.getPostExecuteCallbackFor(task)
        errorCallback = automaton.getErrorCallbackFor(task)
        executeTaskFunction = automaton.getExecuteTaskFunction(task)
        
        requestContext = self.getRequestContext(
            task=task, executeEnvironment=executeEnvironment
        )
        request = threadpool.WorkRequest(
            executeTaskFunction,
            args = [],
            kwds = requestContext,
            callback = successCallback,
            exc_callback = errorCallback
        )
        task.workRequest(request)
        
        pool = self.automaton.getThreadPoolUsingRequest(request)
        requestContext['thread pool'] = pool

        task.automaton(self.automaton)

        pool.putRequest(request)
        pool.wait()

        return request
    
    # END class PrintObject
    pass


class TestCompositeDefinition(unittest.TestCase):
    
    def setUp(self):
        definition = GeneratePomsetsModule.createPomsetContainingParameterSweep()
        GeneratePomsetsModule.bindParameterSweepDefinitionParameters(definition)
        
        self.definitionToCopy = definition
        return

    def testEq(self):

        newDefinition = copy.copy(self.definitionToCopy)
        
        assert newDefinition == self.definitionToCopy
        
        return

    def testCommands(self):
        """
        This tests for functional performance
        """
        
        # here we have to execute a task for both definitions
        # and assert that output of the task for the copy
        # is the same as the output of the task for the original
        
        newDefinition = copy.copy(self.definitionToCopy)
        
        commands = self.buildCommandsToExecute(self.definitionToCopy)
        commandsForCopiedNode = self.buildCommandsToExecute(newDefinition)
        
        assert commands == commandsForCopiedNode
        
        return
 
    
    def createTask(self, definition):

        compositeTask = TaskModule.CompositeTask()
        compositeTask.definition(definition)
        taskGenerator = TaskModule.NestTaskGenerator()
        compositeTask.taskGenerator(taskGenerator)
        compositeTask.setParameterBinding(
            'input file',
            self.inputFiles
        )
        compositeTask.setParameterBinding(
            'output file',
            self.outputFiles
        )
        compositeTask.setParameterBinding(
            'intermediate file',
            self.intermediateFiles
        )
        return compositeTask
    
    def buildCommandsToExecute(self, definition):
        
        testObj = PrintObject()
        
        task = testObj.createTask(definition)
        request = testObj.executeTask(task)

        commands = testObj.env.outputStream().getvalue()
        
        return commands
    
    # END class TestCompositeDefinition
    pass


class TestReferenceDefinition(unittest.TestCase):
    
    def setUp(self):
        definition = GeneratePomsetsModule.createPomsetContainingParameterSweep()
        GeneratePomsetsModule.bindParameterSweepDefinitionParameters(definition)
        
        self.definitionToCopy = definition
        return


    def testCopyMapper(self):
        
        nodeToCopy = [x for x in self.definitionToCopy.nodes()
                      if x.id() == 'mapper'][0]
        
        copiedNode = copy.copy(nodeToCopy)

        self.assertTrue(copiedNode.graph() is None)
        copiedNode.graph(nodeToCopy.graph())

        # assert nodeToCopy == copiedNode
        differences = [x for x in nodeToCopy.getDifferencesWith(copiedNode)]
        self.assertEquals(['self has 1 outgoing edges, other has 0'], 
                          differences)
        
        return


    def testCopyReducer(self):
        nodeToCopy = [x for x in self.definitionToCopy.nodes()
                      if x.id() == 'reducer'][0]
        
        copiedNode = copy.copy(nodeToCopy)

        self.assertTrue(copiedNode.graph() is None)
        copiedNode.graph(nodeToCopy.graph())

        # assert nodeToCopy == copiedNode
        differences = [x for x in nodeToCopy.getDifferencesWith(copiedNode)]
        self.assertEquals(['self has 1 incoming edges, other has 0'],
                          differences)

        
        return
        
    # END class TestReferenceDefinition
    pass


def main():
    # utils.configLogging()

    suite = unittest.TestSuite()
    
    suite.addTest(unittest.makeSuite(TestReferenceDefinition, 'test'))
    suite.addTest(unittest.makeSuite(TestCompositeDefinition, 'test'))
    
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




from __future__ import with_statement

import os
import sys
import unittest

import StringIO

import currypy

import pypatterns.filter as FilterModule

import pomsets.command as ExecuteEnvironmentModule

import TestExecute as BaseModule

class TestCase1(BaseModule.TestCase1):
    """
    execute of atomic function
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    def assertPostExecute(self):
        expected = """echo
/bin/echo foo
"""
        actual = self.env.outputStream().getvalue()
        assert expected == actual, \
               'expected "%s", got "%s"' % (expected, actual)
        return
    
    
    # END TestCase1
    pass


class TestCase2(BaseModule.TestCase2):
    """
    execute of atomic function
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env
    
    def assertPostExecute(self):
        expected = """echo
/bin/echo "echoed testExecuteAtomicFunction2"
"""
        actual = self.env.outputStream().getvalue()
        assert expected == actual, \
               'expected "%s", got "%s"' % (expected, actual)
        return
    
    # END class TestCase2
    pass




class TestCase4(BaseModule.TestCase4):
    """
    execute of composite function
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    def assertPostExecute(self):
        expected = """echo
/bin/echo foo
"""
        actual = self.env.outputStream().getvalue()
        assert expected == actual, \
               'expected "%s", got "%s"' % (expected, actual)
        return

    # END class TestCase4
    pass



class TestCase8(BaseModule.TestCase8):
    """
    execute of composite function
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    def assertPostExecute(self):
        expected = """echo 0
/bin/echo foo
echo 1
/bin/echo foo
echo 2
/bin/echo foo
"""
        actual = self.env.outputStream().getvalue()
        assert expected == actual, \
               'expected "%s", got "%s"' % (expected, actual)
        return
    
    # END class TestCase8
    pass


class TestCase9(BaseModule.TestCase9):
    """
    execute of composite function
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env
    
    def assertPostExecute(self):
        expected = """echo 0
/bin/echo foo
echo 1
/bin/echo foo
echo 2
/bin/echo foo
"""
        actual = self.env.outputStream().getvalue()
        assert expected == actual, \
               'expected "%s", got "%s"' % (expected, actual)
        return
    
    # END class TestCase9
    pass


class TestCase10(BaseModule.TestCase10):
    """
    execution fails due to incomplete parameter binding 
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    
    # END class TestCase
    pass



class TestParameterSweep1(BaseModule.TestParameterSweep1):

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    def assertPostExecute(self):
        expected = """echo
/bin/echo "echoed testExecuteParameterSweep1 : 1"
echo
/bin/echo "echoed testExecuteParameterSweep1 : 2"
"""
        actual = self.env.outputStream().getvalue()
        assert expected == actual, \
               'expected "%s", got "%s"' % (expected, actual)
        return
    
    
    # END class TestParameterSweep1
    pass


class TestParameterSweep2(BaseModule.TestParameterSweep2):

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    def assertPostExecute(self):
        expected = """mapper\n%s/resources/testdata/TestExecute/wordcount.py %s/resources/testdata/TestExecute/text1 /tmp/count1\nmapper\n%s/resources/testdata/TestExecute/wordcount.py %s/resources/testdata/TestExecute/text2 /tmp/count2
""" % tuple([os.getcwd()]*4)

        actual = self.env.outputStream().getvalue()
        self.assertEquals(expected, actual)
        return

    
    # END class TestParameterSweep2
    pass


class TestParameterSweep3(BaseModule.TestParameterSweep3):

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env

    def assertPostExecute(self):
        expected = """reducer\n%s/resources/testdata/TestExecute/wordcount_reduce.py -input %s/resources/testdata/TestExecute/count1 %s/resources/testdata/TestExecute/count2 -output /tmp/count_reduce
""" %  tuple([os.getcwd()]*3)

        actual = self.env.outputStream().getvalue()
        self.assertEquals(expected, actual)
        return
    
    # END class TestParameterSweep3
    pass


class TestParameterSweep4(BaseModule.TestParameterSweep4):
    """
    tests combining a mapper with a reducer
    """

    def createExecuteEnvironment(self):
        io = StringIO.StringIO()
        self.env = ExecuteEnvironmentModule.PrintTaskCommand()
        self.env.postfix('\n')
        self.env.outputStream(io)
        return self.env
    
    def assertPostExecute(self):
        expected = """mapper\n%s/resources/testdata/TestExecute/wordcount.py %s/resources/testdata/TestExecute/text1 /tmp/count1\nmapper\n%s/resources/testdata/TestExecute/wordcount.py %s/resources/testdata/TestExecute/text2 /tmp/count2\nreducer\n%s/resources/testdata/TestExecute/wordcount_reduce.py -input /tmp/count1 /tmp/count2 -output /tmp/count_reduce
"""  % tuple([os.getcwd()]*5)

        actual = self.env.outputStream().getvalue()
        self.assertEquals(expected, actual)
        return
    
    # END class TestParameterSweep4
    pass




def main():
    util.configLogging()

    suite = unittest.TestSuite()

    suite.addTest(unittest.makeSuite(TestCase1, 'test'))
    suite.addTest(unittest.makeSuite(TestCase2, 'test'))
    suite.addTest(unittest.makeSuite(TestCase4, 'test'))
    suite.addTest(unittest.makeSuite(TestCase8, 'test'))
    suite.addTest(unittest.makeSuite(TestCase9, 'test'))
    suite.addTest(unittest.makeSuite(TestCase10, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep1, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep2, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep3, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep4, 'test'))

    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()




import os
import sys

import pomsets.context as ContextModule
import pomsets.library as LibraryModule

import pomsets.test_utils as DefinitionModule


def main(argv=None):

    if argv is None:
        argv = []
        
    if len(argv) < 2:
        raise ValueError('need to specify directory to output the definitions')
    outputDir = argv[1]
    
    baseDefinition = DefinitionModule.createPomsetContainingParameterSweep()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'mr_wordcount.pomset'), baseDefinition)
    DefinitionModule.bindParameterSweepDefinitionParameters(baseDefinition)
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'mr_wordcount_staging.pomset'), baseDefinition)

    
    baseDefinition = DefinitionModule.createPomsetContainingLoopDefinition()
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loop_wordcount.pomset'), baseDefinition)
    DefinitionModule.bindLoopDefinitionParameters(baseDefinition)
    ContextModule.pickleDefinition(
        os.path.join(outputDir, 'loop_wordcount_staging.pomset'), baseDefinition)

    
    return

if __name__=="__main__":
    main(sys.argv)




from __future__ import with_statement

import os
import sys
import threadpool
import unittest
import logging

import pomsets.builder as BuilderModule
import pomsets.command as TaskCommandModule
import pomsets.context as ContextModule
import pomsets.definition as DefinitionModule
import pomsets.library as DefinitionLibraryModule
import pomsets.parameter as ParameterModule
import pomsets.task as TaskModule

ID_WORDCOUNT = 'word count_8613fe86-e7fc-4487-b4d2-0989706f8825'
ID_WORDCOUNT_REDUCE = 'word count reducer_08979d5f-6c0d-43b7-9206-dfe69eae6c26'

def pickleAndReloadDefinition(path, definition):

    # try pickling the definition
    # and the reloading it
    filesToDelete = []
    try:

        ContextModule.pickleDefinition(path, definition)

        filesToDelete.append(path)

        pomsetContext = ContextModule.loadPomset(path)
        definition = pomsetContext.pomset()

    except Exception, e:
        logging.error("errored with msg >> %s" % e)
        pass
    finally:
        for fileToDelete in filesToDelete:
            if os.path.exists(fileToDelete):
                os.unlink(fileToDelete)
                pass
        pass

    return definition


def createWordCountDefinition(dir=None):
    
    parameterOrdering = DefinitionModule.createParameterOrderingTable()
    row = parameterOrdering.addRow()
    row.setColumn('source', 'input file')
    row.setColumn('target', 'output file')

    if dir is None:
        dir = os.getcwd().split(os.path.sep) + ['resources', 'testdata', 'TestExecute']
    command = dir + ['wordcount.py']
    executable = TaskCommandModule.Executable()
    executable.stageable(True)
    executable.path(command)
    executable.staticArgs([])
    
    definition = DefinitionModule.createShellProcessDefinition(
        inputParameters = {
            'input file':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE:True,
            },
            'output file':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:True,
            }
        },
        parameterOrderings = parameterOrdering,
        executable = executable
    )
    definition.name('wordcount mapper')
    definition.id(ID_WORDCOUNT)
    definition.isLibraryDefinition(True)
    
    return definition


DEFINITION_WORDCOUNT = createWordCountDefinition()


def createWordCountReduceDefinition(dir=None):
    
    parameterOrdering = DefinitionModule.createParameterOrderingTable()
    row = parameterOrdering.addRow()
    row.setColumn('source', 'input files')
    row.setColumn('target', 'output file')
    
    if dir is None:
        dir =  os.getcwd().split(os.path.sep) + ['resources', 'testdata',
                                                 'TestExecute']
    command = dir+['wordcount_reduce.py']
    executable = TaskCommandModule.Executable()
    executable.stageable(True)
    executable.path(command)
    executable.staticArgs([])
    
    definition = DefinitionModule.createShellProcessDefinition(
        inputParameters = {
            'input files':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:{
                    ParameterModule.COMMANDLINE_PREFIX_FLAG:['-input']
                    },
                ParameterModule.PORT_ATTRIBUTE_ISINPUTFILE:True,
            },
            'output file':{
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE:True,
                ParameterModule.PORT_ATTRIBUTE_COMMANDLINE_OPTIONS:{
                    ParameterModule.COMMANDLINE_PREFIX_FLAG:['-output']
                },
                ParameterModule.PORT_ATTRIBUTE_ISSIDEEFFECT:True
            }
        },
        parameterOrderings = parameterOrdering,
        executable = executable
    )
    
    definition.name('wordcount reducer')
    definition.id(ID_WORDCOUNT_REDUCE)
    definition.isLibraryDefinition(True)
    
    return definition

DEFINITION_WORDCOUNT_REDUCE = createWordCountReduceDefinition()





def createNestDefinition():
    compositeDefinition = DefinitionModule.getNewNestDefinition()
    return compositeDefinition

def createBranchDefinition():
    raise NotImplementedError


def createPomsetContainingBranchDefinition():
    compositeDefinition = DefinitionModule.getNewNestDefinition()

    branchDefinition = DefinitionModule.BranchDefinition()
    
    # the loop definition needs to add one (or more nodes)
    wcNode = branchDefinition.createNode(id='wordcount')
    wcNode.definitionToReference(DEFINITION_WORDCOUNT)
    wcNode.name('wordcount')
    
    reduceNode = branchDefinition.createNode(id='reduce')
    reduceNode.definitionToReference(DEFINITION_WORDCOUNT_REDUCE)
    reduceNode.name('reduce')
    
    branchNode = compositeDefinition.createNode(id='branch')
    branchNode.definitionToReference(branchDefinition)

    return compositeDefinition


def bindBranchDefinitionParameters(definition):
    nodes = [x for x in definition.nodes() if x.id() == 'branch']
    branchNode = nodes[0]

    branchNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_CONDITION_STATE, 
        0)

    branchNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_CONDITION_FUNCTION, 
        "lambda x: x")

    branchNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_CONDITION_MAP, 
        "[(0, 'wordcount'), (1, 'reduce')]")
    
    return


def createPomsetContainingLoopDefinition():
    compositeDefinition = DefinitionModule.getNewNestDefinition()

    loopDefinition = DefinitionModule.LoopDefinition()
    
    # the loop definition needs to add one (or more nodes)
    wcNode = loopDefinition.createNode(id='wordcount')
    wcNode.definitionToReference(DEFINITION_WORDCOUNT)
    wcNode.name('wordcount')

    loopNode = compositeDefinition.createNode(id='loop')
    loopNode.definitionToReference(loopDefinition)
    
    
    return compositeDefinition


def bindLoopDefinitionParameters(definition):
    nodes = [x for x in definition.nodes() if x.id() == 'loop']
    loopNode = nodes[0]

    # TODO:
    # set the parameter bindings on loopNode
    loopNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_INITIAL_STATE, 
        0)
    loopNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_CONTINUE_CONDITION,
        "lambda x: x < 5")
    loopNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_STATE_TRANSITION,
        "lambda x: x+1"
    )
    loopNode.setParameterBinding(
        DefinitionModule.LoopDefinition.PARAMETER_STATE_CONFIGURATION,
        # set the value of input file
        # set the value of output file
        [
            # for some reason
            # exec is missing
            # we we need to do this using multiple strings
            "childTask.setParameterBinding('input file', ['/tmp/loop' + str(parentTask.getParameterBinding(DefinitionModule.LoopDefinition.PARAMETER_STATE))])",
            "childTask.setParameterBinding('output file', ['/tmp/loop' +  str(parentTask.getParameterBinding(DefinitionModule.LoopDefinition.PARAMETER_STATE)+1)])"
         ]
    )
    return




def createPomsetContainingParameterSweep():

    compositeDefinition = DefinitionModule.getNewNestDefinition()

    # setup the reference definition for parameter sweep
    mapperNode = compositeDefinition.createNode(id='mapper')
    mapperNode.definitionToReference(DEFINITION_WORDCOUNT)
    mapperNode.isParameterSweep('input file', True)
    mapperNode.isParameterSweep('output file', True)
    mapperNode.addParameterSweepGroup(['input file', 'output file'])
    mapperNode.isCritical(True)
    mapperNode.name('mapper')

    reducerNode = compositeDefinition.createNode(id='reducer')
    reducerNode.definitionToReference(DEFINITION_WORDCOUNT_REDUCE)
    reducerNode.isCritical(True)
    reducerNode.name('reducer')

    """
    blackboardParameter = \
        ParameterModule.BlackboardParameter('intermediate file')
    compositeDefinition.addParameter(blackboardParameter)
    compositeDefinition.connectParameters(
        compositeDefinition, 'intermediate file',
        mapperNode, 'output file'
    )
    compositeDefinition.connectParameters(
        compositeDefinition, 'intermediate file',
        reducerNode, 'input files',
    )

    compositeDefinition.connectParameters(
        mapperNode, 'temporal output',
        reducerNode, 'temporal input',
    )
    """
    builder = BuilderModule.Builder()
    builder.connect(
        compositeDefinition,
        mapperNode, 'output file',
        reducerNode, 'input files')
    builder.connect(
        compositeDefinition,
        mapperNode, 'temporal output',
        reducerNode, 'temporal input',
        )

    compositeDefinition.name('basic map-reduce')
    
    return compositeDefinition


def bindParameterSweepDefinitionParameters(definition):
    
    # now we add additional info
    # - file staging flags
    # - parameter values
    nodes = [x for x in definition.nodes() if x.id() == 'mapper']
    mapperNode = nodes[0]

    nodes = [x for x in definition.nodes() if x.id() == 'reducer']
    reducerNode = nodes[0]

    (dataNode, parameterToEdit) = \
     mapperNode.getParameterToEdit('input file')
    dataNode.setParameterBinding(parameterToEdit.id(),
                                 ['/tmp/pomsets/text1', '/tmp/pomsets/text2'])

    (dataNode, parameterToEdit) = \
     mapperNode.getParameterToEdit('output file')

    valuesToBind = ['/tmp/pomsets/count1', '/tmp/pomsets/count2']
    dataNode.setParameterBinding(parameterToEdit.id(),
                                 valuesToBind)


    (dataNode, parameterToEdit) = \
     reducerNode.getParameterToEdit('output file')
    dataNode.setParameterBinding(parameterToEdit.id(), ['/tmp/pomsets/count_all'])
    
    # write out to a different location
    mapperNode.parameterStagingRequired('input file', True)
    mapperNode.parameterStagingRequired('output file', True)
    reducerNode.parameterStagingRequired('input files', True)
    reducerNode.parameterStagingRequired('output file', True)
    
    return definition
    





from __future__ import with_statement

import os
import sys
import threadpool
import unittest
import logging

import paramiko
import simplejson as ConfigModule

#import util
#util.setPythonPath()

import currypy

import cloudpool.shell as ShellModule

import pypatterns.filter as FilterModule

import pomsets.command as CommandModule
import pomsets.definition as DefinitionModule
import pomsets.parameter as ParameterModule
import pomsets.task as TaskModule


import TestExecute as BaseModule


class Credentials(object):
    def loadCredentials(self):

        configFilePath = os.path.join(
            os.getcwd(), 'resources', 'testdata', 'TestExecuteRemote', 'config')

        with open(configFilePath) as f:

            config = ConfigModule.load(f)

            remoteExecuteCredentials = config['remote execute credentials']
            assert len(remoteExecuteCredentials), 'expected to find remote execute credentials'

            return remoteExecuteCredentials

        raise NotImplemented(
            'could not read credentials from config file %s' % configFilePath)


    def getShell(self):

        shell = ShellModule.SecureShell()
        # set the hostname, user, and keyfile

        credentials = self.loadCredentials()

        credential = credentials[0]

        hostname = credential['hostname']
        user = credential['user']
        keyfile = credential['keyfile']

        shell.hostname(hostname)
        shell.user(user)
        shell.keyfile(keyfile)

        return shell

    # ENC class Credentials
    pass

class TestConnection(unittest.TestCase):
    """
    """

    def getCredentialClass(self):
        return Credentials

    def testConnect(self):

        cClass = self.getCredentialClass()
        c = cClass()
        shell = c.getShell()

        shell.establishConnection()

        shell.disconnect()

        return



    # END TestConnection
    pass


class TestCase1(BaseModule.TestCase1):
    """
    execute of atomic function
    """

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestCase1.setUp(self)

        # TODO:
        # use boto to start up an aws VM

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestCase1.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END TestCase1
    pass


class TestCase2(BaseModule.TestCase2):
    """
    execute of atomic function
    """

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestCase2.setUp(self)

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestCase2.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestCase2
    pass




class TestCase4(BaseModule.TestCase4):
    """
    execute of composite function
    """

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestCase4.setUp(self)

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestCase4.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell


    # END class TestCase4
    pass



class TestCase8(BaseModule.TestCase8):
    """
    execute of composite function
    """

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestCase8.setUp(self)

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestCase8.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestCase8
    pass


class TestCase9(BaseModule.TestCase9):
    """
    execute of composite function
    """

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestCase9.setUp(self)

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestCase9.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestCase9
    pass


class TestCase10(BaseModule.TestCase10):
    """
    execution fails due to incomplete parameter binding 
    """

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestCase10.setUp(self)

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestCase10.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestCase
    pass



class TestParameterSweep1(BaseModule.TestParameterSweep1):

    def getCredentialClass(self):
        return Credentials

    def setUp(self):
        BaseModule.TestParameterSweep1.setUp(self)

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()
        return

    def tearDown(self):
        BaseModule.TestParameterSweep1.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestParameterSweep1
    pass


class TestParameterSweep2(BaseModule.TestParameterSweep2):

    BASE_DIR = os.path.sep + os.path.join('tmp', 'TestExecuteRemote', 'TestParameterSweep2')

    def removeFile(self, file):
        try:
            self.fs.remove(file)
        except IOError:
            pass
        return

    def fileExists(self, file):
        try:
            self.fs.open(file)
        except IOError:
            return False
        return True


    def getCredentialClass(self):
        return Credentials

    def setUp(self):

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()

        self.fs = self.shell.getFS()	

        BaseModule.TestParameterSweep2.setUp(self)

        self.stageFiles()
        return


    def stageFiles(self):
        pathParts = TestParameterSweep2.BASE_DIR.split(os.path.sep)[2:]
        # assumes /tmp is already created
        currentPath = os.path.sep + 'tmp'
        for part in pathParts:
            currentPath = os.path.join(currentPath, part)
            if self.fileExists(currentPath):
                continue
            self.fs.mkdir(currentPath)

        inputFilesRemote = [
            os.path.join(x, y)
            for x,y in zip(
                [TestParameterSweep2.BASE_DIR]*len(BaseModule.TestParameterSweep2.INPUT_FILES), 
                BaseModule.TestParameterSweep2.INPUT_FILES)
        ]
        for localFile, remoteFile in zip(self.inputFiles, inputFilesRemote):
            self.fs.put(localFile, remoteFile)
        self.inputFiles = inputFilesRemote

        return

    def removeFiles(self):

        # this only removed the input files
        BaseModule.TestParameterSweep2.removeFiles(self)

        for inputFile in self.inputFiles:
            self.removeFile(inputFile)
            pass

        # path to not remove
        # assumes /tmp was already created
        pathToKeep = os.path.sep + 'tmp'
        currentPath = TestParameterSweep2.BASE_DIR
        while len(currentPath) and not currentPath == pathToKeep:
            self.fs.rmdir(currentPath)
            currentPath = currentPath[:currentPath.rfind(os.path.sep)]

        return

    def tearDown(self):
        BaseModule.TestParameterSweep2.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestParameterSweep2
    pass


class TestParameterSweep3(BaseModule.TestParameterSweep3):

    BASE_DIR = os.path.sep + os.path.join('tmp', 'TestExecuteRemote', 'TestParameterSweep3')

    def removeFile(self, file):
        try:
            self.fs.remove(file)
        except IOError:
            pass
        return

    def fileExists(self, file):
        try:
            self.fs.open(file)
        except IOError:
            return False
        return True


    def getCredentialClass(self):
        return Credentials

    def setUp(self):

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()

        self.fs = self.shell.getFS()

        BaseModule.TestParameterSweep3.setUp(self)

        self.stageFiles()
        return

    def stageFiles(self):
        pathParts = TestParameterSweep3.BASE_DIR.split(os.path.sep)[2:]
        # assumes /tmp is already created
        currentPath = os.path.sep + 'tmp'
        for part in pathParts:
            currentPath = os.path.join(currentPath, part)
            self.fs.mkdir(currentPath)

        inputFilesRemote = [
            os.path.join(x, y) 
            for x,y in zip(
                [TestParameterSweep3.BASE_DIR]*len(BaseModule.TestParameterSweep3.INPUT_FILES), 
                BaseModule.TestParameterSweep3.INPUT_FILES)
        ]

        for localFile, remoteFile in zip(self.inputFiles, inputFilesRemote):
            self.fs.put(localFile, remoteFile)
        self.inputFiles = inputFilesRemote


        return

    def removeFiles(self):

        # this only removed the output files
        BaseModule.TestParameterSweep3.removeFiles(self)

        for inputFile in self.inputFiles:
            self.removeFile(inputFile)
            pass

        # path to not remove
        # assumes /tmp was already created
        pathToKeep = os.path.sep + 'tmp'
        currentPath = TestParameterSweep3.BASE_DIR
        while len(currentPath) and not currentPath == pathToKeep:
            self.fs.rmdir(currentPath)
            currentPath = currentPath[:currentPath.rfind(os.path.sep)]

        return

    def tearDown(self):
        BaseModule.TestParameterSweep3.tearDown(self)
        self.shell.disconnect()
        return

    def createExecuteEnvironment(self):
        return self.shell

    # END class TestParameterSweep3
    pass


class TestParameterSweep4(BaseModule.TestParameterSweep4):
    """
    tests combining a mapper with a reducer
    """
    BASE_DIR = os.path.sep + os.path.join('tmp', 'TestExecuteRemote', 'TestParameterSweep4')

    def removeFile(self, file):
        try:
            self.fs.remove(file)
        except IOError:
            pass
        return

    def fileExists(self, file):
        try:
            self.fs.open(file)
        except IOError:
            return False
        return True

    def getCredentialClass(self):
        return Credentials

    def setUp(self):

        cClass = self.getCredentialClass()
        c = cClass()

        self.shell = c.getShell()
        self.shell.establishConnection()

        self.fs = self.shell.getFS()

        BaseModule.TestParameterSweep4.setUp(self)
        self.stageFiles()
        return


    def stageFiles(self):
        pathParts = TestParameterSweep4.BASE_DIR.split(os.path.sep)[2:]
        # assumes /tmp is already created
        currentPath = os.path.sep + 'tmp'
        for part in pathParts:
            currentPath = os.path.join(currentPath, part)
            self.fs.mkdir(currentPath)

        inputFilesRemote = [
            os.path.join(x, y)
            for x,y in zip(
                [TestParameterSweep4.BASE_DIR]*len(BaseModule.TestParameterSweep4.INPUT_FILES), 
                BaseModule.TestParameterSweep4.INPUT_FILES)
        ]
        self.intermediateFiles = [
            os.path.join(x, y)
            for x,y in zip([TestParameterSweep4.TEST_DIR]*len(TestParameterSweep4.INTERMEDIATE_FILES), 
                           TestParameterSweep4.INTERMEDIATE_FILES)
        ]


        for localFile, remoteFile in zip(self.inputFiles, inputFilesRemote):
            self.fs.put(localFile, remoteFile)
        self.inputFiles = inputFilesRemote


        return

    def removeFiles(self):

        # this only removed the output files
        BaseModule.TestParameterSweep4.removeFiles(self)

        for inputFile in self.inputFiles:
            self.removeFile(inputFile)
            pass

        # path to not remove
        # assumes /tmp was already created
        pathToKeep = os.path.sep + 'tmp'
        currentPath = TestParameterSweep4.BASE_DIR
        while len(currentPath) and not currentPath == pathToKeep:
            self.fs.rmdir(currentPath)
            currentPath = currentPath[:currentPath.rfind(os.path.sep)]

        return

    def tearDown(self):
        BaseModule.TestParameterSweep4.tearDown(self)
        self.shell.disconnect()
        return



    def createExecuteEnvironment(self):
        return self.shell

    # END class TestParameterSweep4
    pass




def main():
    util.configLogging()

    suite = unittest.TestSuite()


    suite.addTest(unittest.makeSuite(TestConnection, 'test'))
    suite.addTest(unittest.makeSuite(TestCase1, 'test'))
    suite.addTest(unittest.makeSuite(TestCase2, 'test'))
    suite.addTest(unittest.makeSuite(TestCase4, 'test'))
    suite.addTest(unittest.makeSuite(TestCase8, 'test'))
    suite.addTest(unittest.makeSuite(TestCase9, 'test'))
    suite.addTest(unittest.makeSuite(TestCase10, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep1, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep2, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep3, 'test'))
    suite.addTest(unittest.makeSuite(TestParameterSweep4, 'test'))
    
    runner = unittest.TextTestRunner()
    runner.run(suite)
    return

if __name__=="__main__":
    main()

